import { Todo } from "@src/core/entities/todo";
import { FC, useState } from "react";
import { TodoCheckmark } from "@src/components/todos/todoCheckmark";
import Tippy from "@tippyjs/react";
import Link from "next/link";
import { todosApi } from "@src/infrastructure/apis/todosApi";

interface TodoItemProps {
  todo: Todo;
  updateTodo: (todo: Todo) => void;
  displayProject: boolean;
}

function getColorFromPriority(priority: string) {
  switch (priority) {
    case "p3":
      return "bg-red-500";

    case "p2":
      return "bg-yellow-500";

    case "p1":
      return "bg-green-500";

    default:
      throw "No priority matches";
  }
}

export const TodoItem: FC<TodoItemProps> = (props) => {
  const [isHovering, setIsHovering] = useState(false);
  const [menuIsOpen, setMenuIsOpen] = useState(false);

  const prefetchTodo = todosApi.usePrefetch("getTodoById");

  return (
    <div
      className="py-3 border-b border-gray-300 dark:border-gray-600 relative"
      onMouseEnter={() => {
        if (!props.todo.id.startsWith("temp")) {
          prefetchTodo(props.todo.id);
        }
        setIsHovering(true);
      }}
      onMouseLeave={() => setIsHovering(false)}
      onFocus={() => setIsHovering(true)}
    >
      <div className="flex items-center space-x-4">
        <TodoCheckmark {...props} />
        <div className="flex flex-col md:flex-row flex-grow gap-0.5 md:gap-2 pr-6">
          <Link href={`/todos/${props.todo.id}`} passHref>
            <a className="flex-grow w-full break-all text-sm">
              {props.todo.title}
            </a>
          </Link>
          <div className="flex flex-row justify-between items-end">
            <div className="flex flex-row">
              {props.todo.description && (
                <div className="h-3 w-3 bg-gray-100 dark:border-black border border-gray-300 dark:bg-gray-900 rounded-sm" />
              )}
              {props.todo.priority && (
                <div
                  className={`rounded-md select-none text-xs p-1 text-white font-bold ${getColorFromPriority(
                    props.todo.priority
                  )}`}
                >
                  {props.todo.priority}
                </div>
              )}
              {!props.todo.authorId && (
                <div className="h-3 w-3 bg-red-500 rounded-sm" />
              )}
            </div>
            {props.displayProject && props.todo.project && (
              <div className="text-gray-500 text-xs text-right whitespace-nowrap place-self-end">
                {props.todo.project}
              </div>
            )}
          </div>
        </div>
      </div>
      <Tippy
        placement="left"
        visible={menuIsOpen}
        onClickOutside={() => {
          setMenuIsOpen(false);
        }}
        interactive={true}
        interactiveBorder={20}
        delay={0}
        duration={0}
        content={
          <div
            className="p-4 bg-gray-700 border border-accent-500 rounded-xl flex flex-col gap-4"
            tabIndex={0}
          >
            <button className="hover:bg-accent-500">Delete</button>
            <Link href={`/todos/${props.todo.id}`} passHref>
              <a>Edit</a>
            </Link>
          </div>
        }
      >
        <div
          className={`absolute left-full top-1/2 w-4 h-4  box-content ml-2 p-2 rounded-md hover:bg-gray-300 active:bg-gray-400 transition group`}
          style={{ transform: "translate(0px, -50%)" }}
          onClick={() => setMenuIsOpen(true)}
        >
          {isHovering && (
            <div className="flex flex-col justify-between h-full">
              <div className="h-0.5 bg-gray-300 rounded-full group-hover:bg-gray-700" />
              <div className="h-0.5 bg-gray-300 rounded-full group-hover:bg-gray-700" />
              <div className="h-0.5 bg-gray-300 rounded-full group-hover:bg-gray-700" />
            </div>
          )}
        </div>
      </Tippy>
    </div>
  );
};
